#include <LowPower.h>
#include <Wire.h> 
#include <DHT.h>
#include "RTClib.h"
RTC_DS1307 rtc;
char daysOfTheWeek[7][12] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
DateTime now;

// Vereinbarungen zur Konfiguration von DHT22 bzw. DHT11
#define DHTPIN 2          // Digitalpin mit dem DHT22
#define DHTTYPE DHT22     // Sensor Typ: DHT 22  (oder für DHT 11: DHT11) 
DHT dht(DHTPIN, DHTTYPE); // DHT-Zugriffsobjekt 

#include <Adafruit_BMP280.h>
Adafruit_BMP280 bmp; // I2C

/////////////////////////////////////////////////////////////////
// DEKLARATIONEN SD Card
/////////////////////////////////////////////////////////////////
#define LOGDATEI "TMP36"
#include <SPI.h>
#include "SdFat.h"

// Chip Selector auf Arduino Board
const uint8_t chipSelect = 10;
// Separatorzeichen für Dateiausgabe
const String SEP = " ; ";
// Zugriff auf Dateisystem der SD Card
SdFat sd;
// Log-Datei
SdFile datei;
// Fehlermeldungen im Flash ablegen.
#define error(msg) sd.errorHalt(F(msg))

String datum, uhrzeit; 
float dhtHumidity;
float dhtTemperature;
float luftdruck;
int   old_day = 0;
int   old_min = 0;

void setup()
{
  Serial.begin(115200); 
  Wire.begin(); 
  if (!bmp.begin()) {  
    Serial.println(F("Could not find a valid BMP280 sensor, check wiring!"));
    while (1);
  }
  initSDCardReader();
  dht.begin(); // DHT22 bzw. DHT11 starten
	// Initialize DS3231
  Serial.println("Initialize RTC");;
  if (! rtc.begin()) {
    Serial.println("Couldn't find RTC");
    while (1);
  }
  //rtc.adjust(DateTime(2018, 1, 27, 11, 17, 0));
  if (! rtc.isrunning()) {
    Serial.println("RTC is NOT running!");
    // following line sets the RTC to the date & time this sketch was compiled
    // rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
    // This line sets the RTC with an explicit date & time, for example to set
    // January 21, 2014 at 3am you would call:
    //  rtc.adjust(DateTime(2018, 1, 21, 20, 29, 0));
  }
  else
  { now = rtc.now();
    old_day = now.day();
    old_min = now.minute();
  } 
  printTemp();      // Temperatur + Luftfeuchte messen und Ausgabe an Display
  delay(100);
  printPressure();  // Luftdruck messen und Ausgabe an Display 
  delay(100);
}

void loop(){   
    // Datum und Uhrzeit lesen 
    now = rtc.now();
    if (old_min != now.minute())
    {
    // einmal je Minute Messwerte schreiben
    old_min = now.minute(); 
     // Datum als String zusammensetzen
    datum = "";
    if( now.day() <10) {datum +="0";}; datum +=  now.day(); datum += "."; 
    if( now.month() <10) {datum +="0";}; datum +=  now.month(); datum += "."; 
    datum += now.year(); datum += " ";
     // Uhrzeit als String zusammensetzen
    uhrzeit = "";
    if (now.hour() < 10){ uhrzeit += "0";}; uhrzeit += now.hour(); uhrzeit += ":";
    if (now.minute() < 10){uhrzeit += "0";}; uhrzeit += now.minute(); uhrzeit += ":";
    if (now.second() < 10){uhrzeit += "0";}; uhrzeit += now.second(); 
    Serial.print(datum); 
    delay(100);
    Serial.println(uhrzeit); 
    delay(100);
    printTemp();      // Temperatur + Luftfeuchte messen und Ausgabe an Display
    delay(100);
    printPressure();  // Luftdruck messen und Ausgabe an Display 
    delay(100);
    schreibeMessung(); 
    }
    if (old_day != now.day() )
    {  
     old_day = now.day();
     // letzte Datei schliessen
     dateiSchliessen(); 
     // neue Datei öffen
     initSDCardReader();
    }
    delay1(8000); //8 Sekunden Tiefschlaf
} // end loop

void printPressure() {
    luftdruck = bmp.readPressure() / 100.0;
    Serial.print("Luftdruck: "); 
    Serial.print(luftdruck); Serial.println("hPa");
 }

void printTemp(){
// Feuchtigkeitswert bestimmen
  dhtHumidity = dht.readHumidity();
  // Temperatur in Celsius auslesen
  dhtTemperature = dht.readTemperature();
  // Sinnvolle Ergebnisse ermittelt?
  if (isnan(dhtHumidity) || isnan(dhtTemperature)) {
    Serial.println("Konnte nicht vom DHT-Sensor lesen.");
    return;
  }
  Serial.print("Feuchtigkeit: "); 
  Serial.print(dhtHumidity); Serial.println(" %\t");
  Serial.print("Temperatur: "); 
  Serial.print(dhtTemperature); Serial.println(" C ");
}

/////////////////////////////////////////////////////////////////
//
// initSDCardReader()
//    Kartenleser initialisieren
//
/////////////////////////////////////////////////////////////////
void initSDCardReader()
{
  const uint8_t NAMENSLAENGE= sizeof(LOGDATEI) - 1;
  char dateiName[13] = LOGDATEI "00.csv"; // Z.B. TMP3604.csv
 
  delay(1000);
  // SD Card mit SPI_HALF_SPEED initialisieren, um Fehler
  // bei Breadboardnutzung zu vermeiden.  Sonst => SPI_FULL_SPEED 
  if (!sd.begin(chipSelect, SPI_HALF_SPEED)) { // Zugriff auf SD?
    sd.initErrorHalt();
  }
 
  // Dateiformat 8.3
  if (NAMENSLAENGE > 6) {
    error("Dateipräfix zu lang");
  }

  // Standarddateiname LOGDATEI  + laufende Nummer, z.B.
  // TMP3603.csv
  // Sobald alle Suffixe 00..09 verbraucht sind,
  // geht es von vorne los: round robin
 
  while (sd.exists(dateiName)) {
    if (dateiName[NAMENSLAENGE + 1] != '9') {
     dateiName[NAMENSLAENGE + 1]++;
    }
    else if (dateiName[NAMENSLAENGE] != '9') {
      dateiName[NAMENSLAENGE + 1] = '0';
      dateiName[NAMENSLAENGE]++;
    }
    else {
      error("Kann Datei nicht erzeugen");
    }
  }
  // Jetzt öffnen:
  if (!datei.open(dateiName, O_CREAT | O_WRITE | O_EXCL)) {
    error("Datei öffnen misslungen!");
  }

  Serial.print(F("Logging auf: "));
  Serial.println(dateiName);

  // Header schreiben
  schreibeHeader();
}

/////////////////////////////////////////////////////////////////
//
// schreibeHeader
//    Schreiben von Header Information in Datei
//
/////////////////////////////////////////////////////////////////
void schreibeHeader() {
  datei.println(F(" Datenlogger mit DHT22 und BMP280"));
  datei.print(F("Datum")); 
  datei.print(SEP);
  datei.print(F("Zeit")); 
  datei.print(SEP);
  datei.print(F("Temperatur"));
  datei.print(SEP);
  datei.print(F("Luftfeuchte"));
  datei.print(SEP);
  datei.print(F("Luftdruck"));
  datei.println();
}

/////////////////////////////////////////////////////////////////
//
// schreibeMessung()
//    messwert nehmen und in gewählter Einheit ausgeben
//    und zusammen mit Datum und Uhrzeit in Datei schreiben
//
/////////////////////////////////////////////////////////////////
void schreibeMessung()
{
  datei.print(datum);
  datei.print(SEP);
  datei.print(uhrzeit);
  datei.print(SEP);
  datei.print(dhtTemperature);
  datei.print(SEP);
  datei.print(dhtHumidity);
  datei.print(SEP);
  datei.print(luftdruck);
  datei.println();
   // Dateisync, um Datenverlust zu vermeiden:
  if (!datei.sync() || datei.getWriteError()) {
    error("Schreibfehler!");
  }
}

/////////////////////////////////////////////////////////////////
//
// dateiSchliessen()
//    Datei wieder schließen
//
/////////////////////////////////////////////////////////////////
void dateiSchliessen()
{
  datei.close();
  Serial.println(F("Datei steht bereit!"));
  // SysCall::halt();
}
/*
 * Versetzt den Prozessort für eine angegebene Zeitspanne ins Tiefschlaf.
 * Nach dem Auswecken wird geprüft, ob die gewünschte Zeit bereits abgelaufen ist
 * und wird ggf. wieder Schlafmodus aktiviert.
 * Man sollte bei der Dauer nicht mit einer besonderen Genauigkeit rechnen.
 */
void delay1(int milliseconds) {
  while (milliseconds >= 8000) { sleep(SLEEP_8S); milliseconds -= 8000; }
  if (milliseconds >= 4000)    { sleep(SLEEP_4S); milliseconds -= 4000; }
  if (milliseconds >= 2000)    { sleep(SLEEP_2S); milliseconds -= 2000; }
  if (milliseconds >= 1000)    { sleep(SLEEP_1S); milliseconds -= 1000; }
  if (milliseconds >= 500)     { sleep(SLEEP_500MS); milliseconds -= 500; }
  if (milliseconds >= 250)     { sleep(SLEEP_250MS); milliseconds -= 250; }
  if (milliseconds >= 125)     { sleep(SLEEP_120MS); milliseconds -= 120; }
  if (milliseconds >= 64)      { sleep(SLEEP_60MS); milliseconds -= 60; }
  if (milliseconds >= 32)      { sleep(SLEEP_30MS); milliseconds -= 30; }
  if (milliseconds >= 16)      { sleep(SLEEP_15MS); milliseconds -= 15; }
}

/*
 * Verwendet LowPower-Bibliothek zum Aktivieren des Schlafmodus der CPU.
 */
void sleep(period_t period) {
  LowPower.powerDown(period, ADC_OFF, BOD_OFF); 
}


